@@ -10,7 +10,7 @@ import html
 from collections import namedtuple, OrderedDict
 import math
 
-__version__ = '0.18.2'
+__version__ = '0.18.3'
 __author__ = 'Danny Lin'
 __author_email__ = 'danny0838@gmail.com'
 __homepage__ = 'https://github.com/danny0838/sts-lib'
@@ -201,7 +201,13 @@ class StsDict():
             return False
         for key, value in self.items():
             try:
-                if not value == other[key]:
+                if value != other[key]:
+                    return False
+            except KeyError:
+                return False
+        for key, value in other.items():
+            try:
+                if value != self[key]:
                     return False
             except KeyError:
                 return False
@@ -398,16 +404,17 @@ class StsDict():
         table:
             註冊表 => 登錄檔
         stsdict:
-            注 => 注 註
+            注 => 註 注
         swapped stsdict:
-            注 => 注
             註 => 注
         table._join_prefix(stsdict):
             注冊表 => 登錄檔
             註冊表 => 登錄檔
         """
         dict_ = self.__class__()
-        converter = stsdict.swap()
+        converter = self.__class__()
+        for key, values in stsdict.items():
+            converter.add(values[0], [key])
         for key, values in self.items():
             for newkey in converter.apply_enum(key, include_short=True, include_self=True):
                 dict_.add(newkey, values)
@@ -918,16 +925,25 @@ class StsListMaker():
     DEFAULT_DICTIONARY_DIR = os.path.join(os.path.dirname(__file__), 'data', 'dictionary')
 
 class StsConverter():
-    """Convert a text using a listfile.
+    """Convert a text using an stsdict.
     """
-    def __init__(self, listfile, options={}):
-        _, ext = os.path.splitext(listfile)
-        if ext.lower() == '.jlist':
-            self.table = Table().loadjson(listfile)
-        elif ext.lower() == '.tlist':
-            self.table = Trie().loadjson(listfile)
-        else: # default: list
-            self.table = Table().load(listfile)
+    def __init__(self, stsdict, options={}):
+        """Initialize a converter.
+
+        Args:
+            stsdict: an StsDict or a str, bytes or os.PathLike object for a
+                dictionary file
+        """
+        if isinstance(stsdict, StsDict):
+            self.table = stsdict
+        else:
+            _, ext = os.path.splitext(stsdict)
+            if ext.lower() == '.jlist':
+                self.table = Table().loadjson(stsdict)
+            elif ext.lower() == '.tlist':
+                self.table = Trie().loadjson(stsdict)
+            else: # default: list
+                self.table = Table().load(stsdict)
         self.options = {**self.default_options, **options}
 
     def convert(self, text):
