@@ -3,6 +3,7 @@ import os
 import re
 import json
 import time
+from pathlib import Path
 from sts import StsListMaker, StsConverter, Unicode, StsDict, Table, Trie
 
 root_dir = os.path.dirname(__file__)
@@ -279,10 +280,15 @@ class TestClassStsDict(unittest.TestCase):
 
     def test_prefix(self):
         for stsdict in self.prepare_dicts({'註冊表': ['登錄檔']}):
-            stsdict2 = Table({'注': ['注', '註']})
+            stsdict2 = Table({'注': ['註', '注' ]})
             stsdict = stsdict._join_prefix(stsdict2)
             self.assertEqual(stsdict, {'注冊表': ['登錄檔'], '註冊表': ['登錄檔']})
 
+        for stsdict in self.prepare_dicts({'註冊表': ['登錄檔']}):
+            stsdict2 = Table({'注': ['注', '註'], '册': ['冊'], '注册': ['註冊']})
+            stsdict = stsdict._join_prefix(stsdict2)
+            self.assertEqual(stsdict, {'注册表': ['登錄檔'], '註冊表': ['登錄檔'], '註册表': ['登錄檔']})
+
     def test_postfix(self):
         for stsdict in self.prepare_dicts({'因为': ['因爲']}):
             stsdict2 = Table({'爲': ['為']})
@@ -299,8 +305,30 @@ class TestClassStsDict(unittest.TestCase):
                 '正則表達式': ['正規表示式'], '正则表达式': ['正規表示式'], '正则表達式': ['正規表示式'], '正則表达式': ['正規表示式']
                 })
 
+        for stsdict in self.prepare_dicts({
+            '万用字元': ['萬用字元'], '数据': ['數據'],
+            '万': ['萬', '万'], '数': ['數'], '据': ['據', '据'], '问': ['問'], '题': ['題'],
+            }):
+            stsdict2 = Table({'元數據': ['後設資料'], '數據': ['資料']})
+            stsdict = stsdict.join(stsdict2)
+            self.assertEqual(stsdict, {
+                '万用字元': ['萬用字元'], '数据': ['資料'],
+                '万': ['萬', '万'], '数': ['數'], '据': ['據', '据'], '问': ['問'], '题': ['題'],
+                '元數據': ['後設資料'], '數據': ['資料'],
+                '元数据': ['後設資料'], '元数據': ['後設資料'], '元數据': ['後設資料'],
+                '数據': ['資料'], '數据': ['資料'],
+                })
 
-class TestSts(unittest.TestCase):
+        for stsdict in self.prepare_dicts({'妳': ['你', '奶']}):
+            stsdict2 = Table({'奶媽': ['奶娘']})
+            stsdict = stsdict.join(stsdict2)
+            self.assertEqual(stsdict, {
+                '妳': ['你', '奶'],
+                '奶媽': ['奶娘'],
+                })
+
+
+class TestClassStsListMaker(unittest.TestCase):
     def convert_text(self, text, config, options={}, method='convert_text'):
         stsdict = StsListMaker().make(config, quiet=True)
         converter = StsConverter(stsdict, options)
@@ -320,87 +348,319 @@ class TestSts(unittest.TestCase):
         result = self.convert_text(input, config or os.path.join(dir, name + '.json'), options)
         self.assertEqual(result, answer)
 
-
-class TestDict(TestSts):
     def test_merge1(self):
-        self.check_case('test_dict', 'merge1')
+        self.check_case('test_make', 'merge1')
 
     def test_merge2(self):
-        self.check_case('test_dict', 'merge2')
+        self.check_case('test_make', 'merge2')
 
     def test_join1(self):
-        self.check_case('test_dict', 'join1')
+        self.check_case('test_make', 'join1')
 
     def test_join2(self):
-        self.check_case('test_dict', 'join2')
+        self.check_case('test_make', 'join2')
 
     def test_join3(self):
-        self.check_case('test_dict', 'join3')
+        self.check_case('test_make', 'join3')
 
     def test_join4(self):
-        self.check_case('test_dict', 'join4')
+        self.check_case('test_make', 'join4')
 
 
-class TestIds(TestSts):
-    def test_ids1(self):
-        self.check_case('test_ids', 'ids1')
+class TestClassStsConverter(unittest.TestCase):
+    def test_init(self):
+        # file as str (.list)
+        tempfile = os.path.join(root_dir, f"test-{time.time()}.list")
+        try:
+            with open(tempfile, 'w', encoding='UTF-8') as f:
+                f.write("""干\t幹 乾 干\n干姜\t乾薑""")
+                f.close()
+            converter = StsConverter(tempfile)
+            self.assertEqual(converter.table, {'干': ['幹', '乾', '干'], '干姜': ['乾薑']})
+        except:
+            raise
+        finally:
+            try:
+                os.remove(tempfile)
+            except FileNotFoundError:
+                pass
 
-    def test_ids2(self):
-        self.check_case('test_ids', 'ids2')
+        # file as str (.jlist)
+        tempfile = os.path.join(root_dir, f"test-{time.time()}.jlist")
+        try:
+            with open(tempfile, 'w', encoding='UTF-8') as f:
+                f.write("""{"干": ["干", "榦"], "姜": ["姜", "薑"], "干姜": ["乾薑"]}""")
+                f.close()
+            converter = StsConverter(tempfile)
+            self.assertEqual(converter.table, {'干': ['干', '榦'], '姜': ['姜', '薑'], '干姜': ['乾薑']})
+        except:
+            raise
+        finally:
+            try:
+                os.remove(tempfile)
+            except FileNotFoundError:
+                pass
 
-    def test_ids_broken1(self):
-        self.check_case('test_ids', 'ids_broken1', 'tw2s')
+        # file as str (.tlist)
+        tempfile = os.path.join(root_dir, f"test-{time.time()}.tlist")
+        try:
+            with open(tempfile, 'w', encoding='UTF-8') as f:
+                f.write("""{"干": {"": ["干", "榦"], "姜": {"": ["乾薑"]}}, "姜": {"": ["姜", "薑"]}}""")
+                f.close()
+            converter = StsConverter(tempfile)
+            self.assertEqual(converter.table, {'干': ['干', '榦'], '姜': ['姜', '薑'], '干姜': ['乾薑']})
+        except:
+            raise
+        finally:
+            try:
+                os.remove(tempfile)
+            except FileNotFoundError:
+                pass
 
-    def test_vi1(self):
-        self.check_case('test_ids', 'vi1')
+        # file as os.PathLike object
+        tempfile = Path(os.path.join(root_dir, f"test-{time.time()}.list"))
+        try:
+            with open(tempfile, 'w', encoding='UTF-8') as f:
+                f.write("""干\t幹 乾 干\n干姜\t乾薑""")
+                f.close()
+            converter = StsConverter(tempfile)
+            self.assertEqual(converter.table, {'干': ['幹', '乾', '干'], '干姜': ['乾薑']})
+        except:
+            raise
+        finally:
+            try:
+                os.remove(tempfile)
+            except FileNotFoundError:
+                pass
 
-    def test_vs1(self):
-        self.check_case('test_ids', 'vs1')
+        # StsDict
+        stsdict = Trie({'干': ['幹', '乾', '干'], '姜': ['姜', '薑'], '干姜': ['乾薑']})
+        converter = StsConverter(stsdict)
+        self.assertEqual(converter.table, {'干': ['幹', '乾', '干'], '姜': ['姜', '薑'], '干姜': ['乾薑']})
 
-    def test_cdm1(self):
-        self.check_case('test_ids', 'cdm1')
+    def test_convert(self):
+        stsdict = StsListMaker().make('s2t', quiet=True)
+        converter = StsConverter(stsdict)
+        self.assertEqual(
+            list(converter.convert("""干了 干涉
+⿰虫风需要简转繁
+⿱艹⿰虫风不需要简转繁
+沙⿰虫风也简转繁""")),
+            [(['干', '了'], ['幹了', '乾了']), ' ', (['干', '涉'], ['干涉']), '\n',
+            '⿰虫风', '需', '要', (['简'], ['簡']), (['转'], ['轉']), '繁', '\n',
+            '⿱艹⿰虫风', '不', '需', '要', (['简'], ['簡']), (['转'], ['轉']), '繁', '\n',
+            '沙', '⿰虫风', '也', (['简'], ['簡']), (['转'], ['轉']), '繁'])
 
+    def test_convert_text(self):
+        stsdict = StsListMaker().make('s2t', quiet=True)
+        converter = StsConverter(stsdict)
+        self.assertEqual(
+            converter.convert_text("""干了 干涉
+⿰虫风需要简转繁
+⿱艹⿰虫风不需要简转繁
+沙⿰虫风也简转繁"""),
+            r"""幹了 干涉
+⿰虫风需要簡轉繁
+⿱艹⿰虫风不需要簡轉繁
+沙⿰虫风也簡轉繁""")
 
-class TestExclude(TestSts):
-    def test_exclude1(self):
-        self.check_case('test_exclude', 'exclude1', 's2t', {
-            'exclude': r'-{(?P<return>.*?)}-',
+    def test_convert_file(self):
+        tempfile = os.path.join(root_dir, f"test-{time.time()}.tmp")
+        tempfile2 = os.path.join(root_dir, f"test2-{time.time()}.tmp")
+        try:
+            stsdict = StsListMaker().make('s2t', quiet=True)
+            converter = StsConverter(stsdict)
+
+            with open(tempfile, 'w', encoding='UTF-8') as f:
+                f.write("""干柴烈火 发财圆梦""")
+                f.close()
+            converter.convert_file(tempfile, tempfile2)
+            with open(tempfile2, 'r', encoding='UTF-8') as f:
+                result = f.read()
+                f.close()
+            self.assertEqual(result, """乾柴烈火 發財圓夢""")
+
+            with open(tempfile, 'w', encoding='GBK') as f:
+                f.write("""干柴烈火 发财圆梦""")
+                f.close()
+            converter.convert_file(tempfile, tempfile2, input_encoding='GBK', output_encoding='Big5')
+            with open(tempfile2, 'r', encoding='Big5') as f:
+                result = f.read()
+                f.close()
+            self.assertEqual(result, """乾柴烈火 發財圓夢""")
+        except:
+            raise
+        finally:
+            try:
+                os.remove(tempfile)
+            except FileNotFoundError:
+                pass
+            try:
+                os.remove(tempfile2)
+            except FileNotFoundError:
+                pass
+
+    def test_convert_option_format(self):
+        stsdict = Trie({
+            '⿰虫风': ['𧍯'],
+            '沙⿰虫风': ['沙虱'],
+            '干': ['幹', '乾', '干'],
+            '干涉': ['干涉'],
+            '会': ['會'],
+            '简': ['簡'],
+            '虫': ['蟲'],
+            '转': ['轉'],
+            '错': ['錯'],
+            '风': ['風'],
             })
-
-    def test_exclude2(self):
-        self.check_case('test_exclude', 'exclude2', 's2t', {
-            'exclude': r'<!-->(?P<return>.*?)<-->',
-            })
-
-    def test_exclude3(self):
-        self.check_case('test_exclude', 'exclude3', 's2twp', {
-            'exclude': r'「.*?」',
-            })
-
-
-class TestFormat(TestSts):
-    def test_generator(self):
-        text = """干了 干涉
+        text = r"""干了 干涉
 ⿰虫风需要简转繁
 ⿱艹⿰虫风不需要简转繁
 沙⿰虫风也简转繁"""
+
+        converter = StsConverter(stsdict, options={'format': 'txt'})
         self.assertEqual(
-            list(self.convert_text(text, 's2t', method='convert')), 
-            [(['干', '了'], ['幹了', '乾了']), ' ', (['干', '涉'], ['干涉']), '\n', '⿰虫风', '需', '要', (['简'], ['簡']), (['转'], ['轉']), '繁', '\n', '⿱艹⿰虫风', '不', '需', '要', (['简'], ['簡']), (['转'], ['轉']), '繁', '\n', '沙', '⿰虫风', '也', (['简'], ['簡']), (['转'], ['轉']), '繁'])
+            converter.convert_text(text),
+            r"""幹了 干涉
+𧍯需要簡轉繁
+⿱艹⿰虫风不需要簡轉繁
+沙虱也簡轉繁"""
+            )
 
-    def test_txtm(self):
-        self.check_case('test_format', 'format_txtm', options={'format': 'txtm'})
+        converter = StsConverter(stsdict, options={'format': 'txtm'})
+        self.assertEqual(
+            converter.convert_text(text),
+            r"""{{干->幹|乾|干}}了 {{干涉}}
+{{⿰虫风->𧍯}}需要{{简->簡}}{{转->轉}}繁
+⿱艹⿰虫风不需要{{简->簡}}{{转->轉}}繁
+{{沙⿰虫风->沙虱}}也{{简->簡}}{{转->轉}}繁"""
+            )
 
-    def test_html(self):
-        self.check_case('test_format', 'format_html', options={'format': 'html'})
+        converter = StsConverter(stsdict, options={'format': 'html'})
+        self.assertEqual(
+            converter.convert_text(text),
+            r"""<span class="sts-conv plural atomic"><del>干</del><ins>幹</ins><ins>乾</ins><ins>干</ins></span>了 <span class="sts-conv single exact"><del>干涉</del><ins>干涉</ins></span>
+<span class="sts-conv single atomic"><del>⿰虫风</del><ins>𧍯</ins></span>需要<span class="sts-conv single atomic"><del>简</del><ins>簡</ins></span><span class="sts-conv single atomic"><del>转</del><ins>轉</ins></span>繁
+⿱艹⿰虫风不需要<span class="sts-conv single atomic"><del>简</del><ins>簡</ins></span><span class="sts-conv single atomic"><del>转</del><ins>轉</ins></span>繁
+<span class="sts-conv single"><del>沙⿰虫风</del><ins>沙虱</ins></span>也<span class="sts-conv single atomic"><del>简</del><ins>簡</ins></span><span class="sts-conv single atomic"><del>转</del><ins>轉</ins></span>繁"""
+            )
 
-    def test_json(self):
-        self.check_case('test_format', 'format_json', options={'format': 'json'})
+        converter = StsConverter(stsdict, options={'format': 'json'})
+        self.assertEqual(
+            converter.convert_text(text),
+            r"""[[["干"], ["幹", "乾", "干"]], "了", " ", [["干", "涉"], ["干涉"]], "\n", [["⿰虫风"], ["𧍯"]], "需", "要", [["简"], ["簡"]], [["转"], ["轉"]], "繁", "\n", "⿱艹⿰虫风", "不", "需", "要", [["简"], ["簡"]], [["转"], ["轉"]], "繁", "\n", [["沙", "⿰虫风"], ["沙虱"]], "也", [["简"], ["簡"]], [["转"], ["轉"]], "繁"]"""
+            )
+
+    def test_convert_option_exclude(self):
+        stsdict = StsListMaker().make('s2t', quiet=True)
+        converter = StsConverter(stsdict, options={'exclude': r'-{(?P<return>.*?)}-'})
+        self.assertEqual(converter.convert_text(r"""-{尸}-廿山女田卜"""), r"""尸廿山女田卜""")
+
+        stsdict = StsListMaker().make('s2t', quiet=True)
+        converter = StsConverter(stsdict, options={'exclude': r'<!-->(?P<return>.*?)<-->'})
+        self.assertEqual(converter.convert_text(r"""发财了<!-->财<--><!-->干<-->"""), r"""發財了财干""")
+
+        stsdict = StsListMaker().make('s2twp', quiet=True)
+        converter = StsConverter(stsdict, options={'exclude': r'「.*?」'})
+        self.assertEqual(converter.convert_text(r"""「奔馳」不是奔馳"""), r"""「奔馳」不是賓士""")
+
+
+class TestBasicCases(unittest.TestCase):
+    def test_ids(self):
+        stsdict = Trie({
+            '会': ['會'],
+            '简': ['簡'],
+            '虫': ['蟲'],
+            '转': ['轉'],
+            '错': ['錯'],
+            '风': ['風'],
+            })
+        converter = StsConverter(stsdict)
+        self.assertEqual(converter.convert_text('⿰虫风简转繁不会出错'), '⿰虫风簡轉繁不會出錯')
+        self.assertEqual(converter.convert_text('⿱艹⿰虫风简转繁不会出错'), '⿱艹⿰虫风簡轉繁不會出錯')
+
+        stsdict = Trie({
+            '⿰虫风': ['𧍯'],
+            '会': ['會'],
+            '简': ['簡'],
+            '虫': ['蟲'],
+            '转': ['轉'],
+            '错': ['錯'],
+            '风': ['風'],
+            })
+        converter = StsConverter(stsdict)
+        self.assertEqual(converter.convert_text('⿰虫风需要简转繁'), '𧍯需要簡轉繁')
+        self.assertEqual(converter.convert_text('⿱艹⿰虫风不需要简转繁'), '⿱艹⿰虫风不需要簡轉繁')
+
+        stsdict = Trie({
+            '⿱艹⿰虫风': ['⿱艹𧍯'],
+            '会': ['會'],
+            '简': ['簡'],
+            '虫': ['蟲'],
+            '转': ['轉'],
+            '错': ['錯'],
+            '风': ['風'],
+            })
+        converter = StsConverter(stsdict)
+        self.assertEqual(converter.convert_text('⿰虫风不需要简转繁'), '⿰虫风不需要簡轉繁')
+        self.assertEqual(converter.convert_text('⿱艹⿰虫风需要简转繁'), '⿱艹𧍯需要簡轉繁')
+
+    def test_ids_broken(self):
+        stsdict = StsListMaker().make('tw2s', quiet=True)
+        converter = StsConverter(stsdict)
+        self.assertEqual(converter.convert_text('IDC有這些：⿰⿱⿲⿳⿴⿵⿶⿷⿸⿹⿺⿻，接著繁轉簡'), 'IDC有这些：⿰⿱⿲⿳⿴⿵⿶⿷⿸⿹⿺⿻，接着繁转简')
+        self.assertEqual(converter.convert_text('「⿰⿱⿲⿳」不影響後面'), '「⿰⿱⿲⿳」不影响后面')
+        self.assertEqual(converter.convert_text('⿰⿱⿲⿳⿴⿵⿶⿷⿸⿹⿺⿻長度不夠\n這行無影響'), '⿰⿱⿲⿳⿴⿵⿶⿷⿸⿹⿺⿻長度不夠\n这行无影响')
+
+    def test_vi(self):
+        stsdict = Trie({
+            '劍': ['剑'],
+            '〾劍': ['剑'],
+            '訢': ['欣', '䜣'],
+            '劍訢': ['剑䜣'],
+            })
+        converter = StsConverter(stsdict)
+        self.assertEqual(converter.convert_text('刀劍 劍訢'), '刀剑 剑䜣')
+        self.assertEqual(converter.convert_text('刀〾劍 〾劍訢 劍〾訢 〾劍〾訢'), '刀剑 剑欣 剑〾訢 剑〾訢')
+
+    def test_vs(self):
+        stsdict = Trie({
+            '劍': ['剑'],
+            '劍󠄁': ['剑'],
+            '訢': ['欣', '䜣'],
+            '劍訢': ['剑䜣'],
+            })
+        converter = StsConverter(stsdict)
+        self.assertEqual(converter.convert_text('刀劍 劍訢'), '刀剑 剑䜣')
+        self.assertEqual(converter.convert_text('刀劍󠄁 劍󠄁訢'), '刀剑 剑欣')
+        self.assertEqual(converter.convert_text('刀劍󠄃 劍󠄃訢'), '刀劍󠄃 劍󠄃欣')
+        self.assertEqual(converter.convert_text('刀劍󠄁󠄂 劍󠄁󠄂訢'), '刀劍󠄁󠄂 劍󠄁󠄂欣')
+
+    def test_cdm(self):
+        stsdict = Trie({
+            '黑桃A': ['葵扇A'],
+            '黑桃Å': ['扇子Å'],
+            })
+        converter = StsConverter(stsdict)
+        self.assertEqual(converter.convert_text('出黑桃A'), '出葵扇A')
+        self.assertEqual(converter.convert_text('出黑桃Å'), '出扇子Å')
+        self.assertEqual(converter.convert_text('出黑桃A̧'), '出黑桃A̧')
+        self.assertEqual(converter.convert_text('出黑桃Å̧'), '出黑桃Å̧')
+
+        stsdict = Trie({
+            'A片': ['成人片'],
+            'Å片': ['特製成人片'],
+            })
+        converter = StsConverter(stsdict)
+        self.assertEqual(converter.convert_text('看A片'), '看成人片')
+        self.assertEqual(converter.convert_text('看Å片'), '看特製成人片')
+        self.assertEqual(converter.convert_text('看A̧片'), '看A̧片')
+        self.assertEqual(converter.convert_text('看Å̧片'), '看Å̧片')
 
 
-@unittest.skip
 class TestConfigs(unittest.TestCase):
-    def test_configs(self):
+    @unittest.skip
+    def test_make(self):
         def clear_lists():
             pattern = re.compile(r'\.(?:[jt]?list)$', re.I)
             for fh in os.scandir(config_dir):
